VALID_GAINS = {
    "a": 1.0,
    "b": 4.0,
    "c": 16.0,
    "d": 64.0,
    "e": 256.0,
    "f": 340.0,
    "g": 512.0
}


class PnccdCtrl():
    def __init__(
        self,
        run_dc: "extra_data.DataCollection",  # noqa
        karabo_id: str
    ):
        """ Extract control data from given control paths.
        :param run_dc: run Extra-data data collection.
        :param karabo_id: Detector identifier name.
        """
        self.run_dc = run_dc
        self.ctrl_src = f"{karabo_id}/CTRL/TCTRL"
        self.mdl_src_temp = f"{karabo_id}/MDL/{{}}"

    def get_bias_voltage(self):
        return(
            abs(self.run_dc.get_run_value(
                self.mdl_src_temp.format("DAQ_MPOD"), "u0voltage.value")))

    def get_gain(self):
        return(
            self.run_dc.get_run_value(
                self.mdl_src_temp.format("DAQ_GAIN"), "pNCCDGain.value"))

    def get_fix_temperature_top(self):
        return self.run_dc.get_run_value(self.ctrl_src, "inputA.krdg.value")

    def get_fix_temperature_bot(self):
        return self.run_dc.get_run_value(self.ctrl_src, "inputB.krdg.value")
